@@ -1,14 +1,6 @@
 """
 Produce an optimal partition by solving an integer linear program (ILP).
 
-The top-level solver is CVXPY, but it must have some concrete ILP solvers installed. The options are:
-
-* XPRESS, CPLEX, GUROBI, MOSEK - work well, but they are not free.
-* ECOS_BB - gives wrong results even on simple problems - not recommended.
-* GLPK_MI (comes with CVXOPT) - very slow.
-* SCIP - Requires lengthy installation. See here https://github.com/scipopt/PySCIPOpt for details.
-* CBC - Requires installation: pip install cylp. See here https://github.com/coin-or/CyLP for details.
-
 Programmer: Erel Segal-Halevi
 Since: 2022-02
 
@@ -18,11 +10,9 @@ Credit: Rob Pratt, https://or.stackexchange.com/a/6115/2576
 from typing import List, Callable, Any
 from numbers import Number
 from prtpy import objectives as obj, outputtypes as out
+from math import inf
 
-import cvxpy
-from cvxpy.reductions.solvers import defines as slv_def
-
-DEFAULT_ILP_SOLVER = cvxpy.CBC
+import mip
 
 
 def optimal(
@@ -32,7 +22,7 @@ def optimal(
     objective: obj.Objective = obj.MinimizeDifference,
     outputtype: out.OutputType = out.Partition,
     copies=1,
-    solver=DEFAULT_ILP_SOLVER,
+    max_seconds=inf,
 ):
     """
     Produce a partition that minimizes the given objective,
@@ -50,24 +40,20 @@ def optimal(
     The following examples are based on:
         Walter (2013), 'Comparing the minimum completion times of two longest-first scheduling-heuristics'.
     >>> walter_numbers = [46, 39, 27, 26, 16, 13, 10]
-    >>> optimal(3, walter_numbers, objective=obj.MinimizeDifference, outputtype=out.PartitionAndSums)
+    >>> optimal(3, walter_numbers, objective=obj.MinimizeDifference, outputtype=out.PartitionAndSums).sort()
     Bin #0: [39, 16], sum=55.0
     Bin #1: [46, 13], sum=59.0
     Bin #2: [27, 26, 10], sum=63.0
-    >>> optimal(3, walter_numbers, objective=obj.MinimizeLargestSum, outputtype=out.PartitionAndSums)
+    >>> optimal(3, walter_numbers, objective=obj.MinimizeLargestSum, outputtype=out.PartitionAndSums).sort()
     Bin #0: [27, 26], sum=53.0
-    Bin #1: [39, 13, 10], sum=62.0
-    Bin #2: [46, 16], sum=62.0
-    >>> optimal(3, walter_numbers, objective=obj.MaximizeSmallestSum, outputtype=out.PartitionAndSums)
-    Bin #0: [27, 16, 13], sum=56.0
-    Bin #1: [46, 10], sum=56.0
+    Bin #1: [46, 16], sum=62.0
+    Bin #2: [39, 13, 10], sum=62.0
+    >>> optimal(3, walter_numbers, objective=obj.MaximizeSmallestSum, outputtype=out.PartitionAndSums).sort()
+    Bin #0: [46, 10], sum=56.0
+    Bin #1: [27, 16, 13], sum=56.0
     Bin #2: [39, 26], sum=65.0
     >>> optimal(3, walter_numbers, objective=obj.MaximizeSmallestSum, outputtype=out.SmallestSum)
     56.0
-    >>> optimal(3, walter_numbers, solver="XYZ")
-    Traceback (most recent call last):
-     ...
-    cvxpy.error.SolverError: Solver XYZ is not installed!
 
     >>> from prtpy.partitioning import partition
     >>> partition(algorithm=optimal, numbins=3, items={"a":1, "b":2, "c":3, "d":3, "e":5, "f":9, "g":9})
@@ -75,20 +61,23 @@ def optimal(
     >>> partition(algorithm=optimal, numbins=2, items={"a":1, "b":2, "c":3, "d":3, "e":5, "f":9, "g":9}, outputtype=out.Sums)
     array([16., 16.])
     """
-    if solver not in slv_def.INSTALLED_SOLVERS:
-        raise cvxpy.SolverError(f"Solver {solver} is not installed!")
 
     ibins = range(numbins)
     if isinstance(copies, Number):
         copies = {item: copies for item in items}
 
+    model = mip.Model("partition")
     counts: dict = {
-        item: [cvxpy.Variable(integer=True) for ibin in ibins] for item in items
+        item: [model.add_var(var_type=mip.INTEGER) for ibin in ibins] for item in items
     }  # counts[i][j] determines how many times item i appears in bin j.
     bin_sums = [
         sum([counts[item][ibin] * map_item_to_value(item) for item in items]) for ibin in ibins
     ]
 
+    model.objective = mip.minimize(
+        objective.get_value_to_minimize(bin_sums, are_sums_in_ascending_order=True)        
+    )
+
     # Construct the list of constraints:
     counts_are_non_negative = [counts[item][ibin] >= 0 for ibin in ibins for item in items]
     each_item_in_one_bin = [
@@ -98,27 +87,19 @@ def optimal(
         bin_sums[ibin + 1] >= bin_sums[ibin] for ibin in range(numbins - 1)
     ]
     constraints = counts_are_non_negative + each_item_in_one_bin + bin_sums_in_ascending_order
+    for constraint in constraints: model += constraint
 
     # Solve the ILP:
-    problem = cvxpy.Problem(
-        objective=cvxpy.Minimize(
-            objective.get_value_to_minimize(bin_sums, are_sums_in_ascending_order=True)
-        ),
-        constraints=constraints,
-    )
-    # CVXPY requires you to specify the solver for mixed-integer programs.
-    problem.solve(solver)
-    if problem.status == "infeasible":
-        raise ValueError("Problem is infeasible")
-    elif problem.status == "unbounded":
-        raise ValueError("Problem is unbounded")
-
+    model.verbose = 0
+    status = model.optimize(max_seconds=max_seconds)
+    if status != mip.OptimizationStatus.OPTIMAL:
+        raise ValueError(f"Problem status is not optimal - it is {status}.")
 
     # Construct the output:
     bins = outputtype.create_empty_bins(numbins)
     for ibin in ibins:
         for item in items:
-            count_item_in_bin = int(counts[item][ibin].value)
+            count_item_in_bin = int(counts[item][ibin].x)
             for _ in range(count_item_in_bin):
                 bins.add_item_to_bin(item, map_item_to_value(item), ibin, inplace=True)
     return outputtype.extract_output_from_bins(bins)
